"""
Create by yy on 2019/9/27
"""

__title__ = 'redis-yy'
__description__ = 'A tool which is used to connect redis, designed by yy'
__url__ = 'https://github.com/guaidashu/redis_yy'
__version_info__ = ('0', '0', '5')
__version__ = '.'.join(__version_info__)
__author__ = 'guaidashu'
__author_email__ = 'song42960@gmail.com'
__maintainer__ = 'YY blog'
__license__ = 'MIT'
__copyright__ = '(c) 2019 by guaidashu'
__install_requires__ = [
    "redis <= 3.3.8"
]
